/*
   Siag, Scheme In A Grid
   Copyright (C) 2000-2001  Ulric Eriksson <ulric@siag.nu>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.
 */
 
#include "../config.h"

#ifdef HAVE_LIBSDB

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/time.h>
#include <unistd.h>
 
#include <sdb.h>

#include "../common/cmalloc.h"
#include "../common/common.h"
#include "../siod/siod.h"
#include "calc.h"

struct db_info {
	buffer *buf;
	int sht, row, col;
	LISP res;
};

static int db_callback(int n, char **p, void *closure)
{
	cval val;
	int i, t;
	char *e;
	struct db_info *dbi = (struct db_info *)closure;

	if (!n) return 0;
	for (i = 0; i < n; i++) {
		val.number = strtod(p[i], &e);
		if (*e == '\0') t = MNUMBER;
		else t = MTEXT;
		if (dbi->res == NIL) {
			if (t == MTEXT) {
				dbi->res = strcons(strlen(p[i]), p[i]);
			} else {
				dbi->res = flocons(val.number);
			}
		} else {
			ins_data(dbi->buf, siod_interpreter,
				p[i], val, t,
				dbi->sht, dbi->row, dbi->col+i);
		}
	}
	dbi->row += 1;
	return 0;
}

/*
;@db_query(url, query)
;@Makes database queries. <a href="db.html">More information</a>.
;@db_query("odbc:postgresql", "select * from foo")
;@
*/
static LISP db_query(LISP url, LISP query)
{
	struct db_info dbi;
        char *u = get_c_string(url);
        char *q = get_c_string(query);
 
        if (u == NULL || q == NULL) return NIL;

	get_siod_coords(&(dbi.row), &(dbi.col), &(dbi.sht), &(dbi.buf));
	dbi.res = NIL;
	sdb_query(u, q, db_callback, &dbi);
	return dbi.res;
}


void init_db(void)
{
	sdb_init();
	init_subr_2("db_query", db_query);
}

#else

void init_db(void)
{
	;
}

#endif
